require "kong.tools.ngx_stub"

local jwt_parser = require "kong.plugins.jwt.jwt_parser"

describe("JWT parser", function()
  describe("Encoding", function()
    it("should properly encode", function()
      local token = jwt_parser.encode({
        sub = "1234567890",
        name = "John Doe",
        admin = true
      }, "secret")
      assert.equal("eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJhZG1pbiI6dHJ1ZSwibmFtZSI6IkpvaG4gRG9lIiwic3ViIjoiMTIzNDU2Nzg5MCJ9.eNK_fimsCW3Q-meOXyc_dnZHubl2D4eZkIcn6llniCk", token)
    end)
  end)
  describe("Decoding", function()
    it("should throw an error if not given a string", function()
      assert.has_error(function()
        jwt_parser:new()
      end, "JWT must be a string")
    end)
    it("should refuse invalid typ", function()
      local token = jwt_parser.encode({sub = "1234"}, "secret", nil, {typ = "foo"})
      local _, err = jwt_parser:new(token)
      assert.equal("Invalid typ", err)
    end)
    it("should refuse invalid alg", function()
      local token = jwt_parser.encode({sub = "1234"}, "secret", nil, {typ = "JWT", alg = "foo"})
      local _, err = jwt_parser:new(token)
      assert.equal("Invalid alg", err)
    end)
  end)
  describe("Verify signature", function()
    it("should verify a signature", function()
      local token = jwt_parser.encode({sub = "foo"}, "secret")
      local jwt, err = jwt_parser:new(token)
      assert.falsy(err)
      assert.True(jwt:verify_signature("secret"))
      assert.False(jwt:verify_signature("invalid"))
    end)
  end)
  describe("Verify registered claims", function()
    it("should require claims passed as arguments", function()
      local token = jwt_parser.encode({sub = "foo"}, "secret")
      local jwt, err = jwt_parser:new(token)
      assert.falsy(err)
      local valid, errors = jwt:verify_registered_claims({"exp", "nbf"})
      assert.False(valid)
      assert.same({exp = "must be a number", nbf = "must be a number"}, errors)

      valid, errors = jwt:verify_registered_claims({"nbf"})
      assert.False(valid)
      assert.same({nbf = "must be a number"}, errors)
    end)
    it("should check the type of given registered claims", function()
      local token = jwt_parser.encode({exp = "bar", nbf = "foo"}, "secret")
      local jwt, err = jwt_parser:new(token)
      assert.falsy(err)
      local valid, errors = jwt:verify_registered_claims({"exp", "nbf"})
      assert.False(valid)
      assert.same({exp = "must be a number", nbf = "must be a number"}, errors)
    end)
    it("should check the exp claim", function()
      local token = jwt_parser.encode({exp = os.time()}, "secret")
      local jwt, err = jwt_parser:new(token)
      assert.falsy(err)
      local valid, errors = jwt:verify_registered_claims({"exp"})
      assert.False(valid)
      assert.same({exp = "token expired"}, errors)
    end)
    it("should check the nbf claim", function()
      local token = jwt_parser.encode({nbf = os.time() + 10}, "secret")
      local jwt, err = jwt_parser:new(token)
      assert.falsy(err)
      local valid, errors = jwt:verify_registered_claims({"nbf"})
      assert.False(valid)
      assert.same({nbf = "token not valid yet"}, errors)
    end)
  end)
end)
